/*
 * Copyright (c) 2008-2009 Instrumentation Technologies
 * All Rights Reserved.
 *
 * $Id: version.cpp 17623 2012-09-05 07:58:09Z damijan.skvarc $
 */

/**
    This file contains version info, provided by build infrastructure.
    Each binary file has its own instance of this file linked in.
    This build information is fed into general purpose singleton istd::VerInfo.
*/

#include "istd/verinfo.h"

#define __TO_STR(x)    #x
#define TO_STR(x)    __TO_STR(x)

#ifndef VER_MAJOR
    #warning Missing version info define.
    #define VER_MAJOR   0
#endif

#ifndef VER_MINOR
    #warning Missing version info define.
    #define VER_MINOR   0
#endif

#ifndef VER_BUILD
    #warning Missing version info define.
    #define VER_BUILD   0
#endif

#ifndef REVISION
    #warning Missing version info define.
    #define REVISION    0
#endif

#ifndef VER_LABEL
    #warning Missing version info define.
    #define VER_LABEL   "undefined label"
#endif

#ifndef VER_COMPILERINFO
    #warning Missing version info define.
    #define VER_COMPILERINFO    "unknown compiler"
#endif

#ifndef VER_BUILDMACHINE
    #warning Missing version info define.
    #define VER_BUILDMACHINE    "unknown build host"
#endif

#ifndef VER_BUILDTIME
    #warning Missing version info define.
    #define VER_BUILDTIME       __TIME__
#endif

#ifndef VER_BUILDUSER
    #warning Missing version info define.
    #define VER_BUILDUSER       "unknown build user"
#endif

/* Constants containing version and build information */
static const int c_mcivermaj = VER_MAJOR;
static const int c_mcivermin = VER_MINOR;
static const int c_mciverbuild = VER_BUILD;
static const int c_mciverrev = REVISION;
static const char c_mciverlabel[] = VER_LABEL;

static const char c_mciversionstr[] =
        TO_STR(VER_MAJOR) "." TO_STR(VER_MINOR) "-" TO_STR(VER_BUILD) "-" VER_LABEL;

static const char c_mcibuildcompiler[] = VER_COMPILERINFO;
static const char c_mcibuildmachine[] = VER_BUILDMACHINE;
static const char c_mcibuildtime[] = VER_BUILDTIME;
static const char c_mcibuilduser[] = VER_BUILDUSER;

//
// Version is updated only from the main executable, not from the shared
// libraries.
//
// NOTE: this limitation can be removed when the singleton istd::VerInfo is able
// to store multiple versions.
//
#ifdef EXECUTABLE

class VersionInitialiser {
    public:
        VersionInitialiser() {
            istd::VerInfo::Init(
                c_mcivermaj, c_mcivermin, c_mciverbuild, c_mciverrev,
                c_mciverlabel, c_mcibuildcompiler, c_mcibuildtime,
                c_mcibuildmachine, c_mcibuilduser);
        }
};

// Global instance to set the version from here.
static VersionInitialiser __attribute__((used)) g_versionInitialiser;

#endif

/* Constant strings for parsing the strings from binaries */
static const char __attribute__((used)) c_verinfo_str[] = "$MCIVersion: " \
    TO_STR(VER_MAJOR) "." TO_STR(VER_MINOR) "-" TO_STR(VER_BUILD) "-" \
    VER_LABEL "-r" TO_STR(REVISION) " $";

static const char __attribute__((used)) c_svnrevision_str[] =
        "$MCISvnRevision: " TO_STR(REVISION) " $";
static const char __attribute__((used)) c_buildcompiler_str[] =
        "$MCIBuildCompiler: " VER_COMPILERINFO " $";
static const char __attribute__((used)) c_buildmachine_str[] =
        "$MCIBuildMachine: " VER_BUILDMACHINE " $";
static const char __attribute__((used)) c_buildtime_str[] =
        "$MCIBuildTime: " VER_BUILDTIME " $";
static const char __attribute__((used)) c_builduser_str[] =
        "$MCIBuildUser: " VER_BUILDUSER " $";
